# Juju Countdown Sensor
#
# Version: 0.8
# Author: Juju <juju@independentcoders.net>


from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import commands
import os
import time
import gtk

# Our class (name: Countdown, derived from: Sensor)
class Countdown(Sensor):

	# Initialization
	def __init__(self, *args):

		Sensor.__init__(self)

		# Config fields
		self._set_config_type("countup", TYPE_INT, 0)
		self._set_config_type("year", TYPE_INT, 2004)
		self._set_config_type("month", TYPE_INT, 1)
		self._set_config_type("day", TYPE_INT, 1)
		self._set_config_type("time", TYPE_STRING, "00:00")
		self._set_config_type("bigmessage", TYPE_STRING, "Hurrah!")
		self._set_config_type("smallmessage", TYPE_STRING, "At last!")
		self._set_config_type("icon", TYPE_STRING, "safari3.png")

		# Timer: the updating thread will be run each 200 milliseconds
		self._add_timer(200, self.__my_timer)

		# This is used to display the target date/time when the user clicks on the desklet
		self._show_config = 0
		self._remaining = ""

		# Trick to add clipboard support (need a real graphical object)
		self._window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self._window.set_title("Set Selection")
		self._window.set_border_width(10)
		self._eventbox = gtk.EventBox()
		self._eventbox.selection_add_target("PRIMARY", "STRING", 1)
		self._eventbox.selection_add_target("PRIMARY", "COMPOUND_TEXT", 1)
		self._eventbox.selection_add_target("CLIPBOARD", "STRING", 1)
		self._eventbox.selection_add_target("CLIPBOARD", "COMPOUND_TEXT", 1)
		self._eventbox.connect("selection_get", self.selection_handle)
		self._eventbox.show()
		self._window.add(self._eventbox)
		self._window.show()
		self._window.hide()


	# We are the clipboard owner : set value
	def selection_handle(self, widget, selection_data, info, time_stamp):
		selection_data.set_text(self._remaining, len(self._remaining))


	# Called when building the Configuration window
	def get_configurator(self):

		# General
		configurator = self._new_configurator()
		configurator.set_name("Juju Countdown")

		# Type
		configurator.add_title("Type")
		configurator.add_option("Counter type:", "countup", "Countdown or countup?",
								[("Countdown",0),("Countup",1)])

		# Target settings
		configurator.add_title("Reference date/time")
		configurator.add_spin("Year:", "year", "Target year", 2000, 2100)
		configurator.add_spin("Month:", "month", "Target month", 1, 12)
		configurator.add_spin("Day:", "day", "Target day of the month", 1, 31)
		configurator.add_entry("Time:","time","Format: HH:MM")

		# Messages
		configurator.add_title("Messages when reached ('countdown' only)")
		configurator.add_entry("Main:","bigmessage","Say 'hurrah'!")
		configurator.add_entry("Legend:","smallmessage","Add a comment here")

		# Icon
		configurator.add_title("Icon")
		configurator.add_option("Use this icon:", "icon", "Choose an icon to be displayed",
								[("Watch+Hands","safari1.png"),("Watch+Hands+Tux","safari2.png"),("Watch+Tux","safari3.png")])

		return configurator


	# The updating timer function
	def __my_timer(self):

		# We'll send data to the display
		output = self._new_output()
		
		if(self._show_config > time.mktime(time.localtime())):
			# We are in a "display target date & time" period
			if(self._get_config("countup") == 0):
				output.set("days","Target:")
			else:
				output.set("days","Ref:")
			output.set("time",str(self._get_config("year"))+"-"+str(self._get_config("month"))+"-"+str(self._get_config("day"))+" "+self._get_config("time"))

			# Yeah, let's display another icon :)
			if(self._get_config("icon")=="safari1.png"):
				output.set("icon","gfx/safari2.png")
			else:
				output.set("icon","gfx/safari1.png")
		else:
			# Normal behaviour

			# Our pretty icon
			output.set("icon","gfx/"+self._get_config("icon"))

			# Okay, let's go
			try:
				# Calculate the difference between current and target date/time
				if(self._get_config("countup") == 0):
					difftime = time.mktime(time.strptime(str(self._get_config("year"))+" "+str(self._get_config("month"))+" "+str(self._get_config("day"))+" "+self._get_config("time"),"%Y %m %d %H:%M")) - time.mktime(time.localtime())
				else:
					difftime = time.mktime(time.localtime()) - time.mktime(time.strptime(str(self._get_config("year"))+" "+str(self._get_config("month"))+" "+str(self._get_config("day"))+" "+self._get_config("time"),"%Y %m %d %H:%M"))
			except ValueError:
				# strptime couldn't identify a valid date/time (maybe the user is typing new values in the configuration window?)
				output.set("days","?? days")
				output.set("time","??h ??m ??s")
			else:
				if(difftime <= 0):
					if(self._get_config("countup")==0):
						# We have reached the target
						output.set("days",self._get_config("bigmessage"))
						output.set("time",self._get_config("smallmessage"))
					else:
						# We have not reached the reference yet
						output.set("days","zZzZ...")
						output.set("time","Having a rest!")
				else:
					# Target is not reached
					
					# Calculate remaining time (difftime is in seconds)
					days = int(difftime / (3600*24))
					hours = int((difftime/(3600))%24)
					minutes = int(((difftime-3600*hours)/60)%60)
					seconds = int(difftime%60)

					# Prepare strings that will be displayed
					sd = str(days) + " day"
					if(days > 1):
						sd += "s"

					st = str(hours) + "h "
			
					# Minutes are always displayed on 2 digits...
					if(minutes<10):
						st += "0"
					st += str(minutes) + "m "

					# ...and seconds as well
					if(seconds<10):
						st += "0"
					st += str(seconds) + "s"

					# This is used for the clipboard thing
					self._remaining = sd + " " + st

					output.set("days",sd)
					output.set("time",st)
				
		# Send our values (icon, days, time) to the display
		self._send_output(output)
		return 1


	# Event callback
	def call_action(self, action, path, args = []):
	
		# User has clicked the desklet
		if(action == "click"):
			if(self._show_config <= time.mktime(time.localtime())):
				# Display target date/time during 3 seconds
				self._show_config = time.mktime(time.localtime()) + 3
			else:
				# Already displaying this, so let's set us as clipboard owner
				self._eventbox.selection_owner_set("PRIMARY")
				self._eventbox.selection_owner_set("CLIPBOARD")
		else:
			print(action)


# Required for gDesklets to be able to load ourselves
def new_sensor(args): return apply(Countdown, args)
